@@ -1,73 +1,33 @@
 from lxml import etree as ET
+from lxml.etree import Element
 import os, shutil
 import logging
 import tempfile
 from . import static, utils, generate, core
 from . import build as builder
 from pathlib import Path
+import sys
 from typing import Optional
 
 log = logging.getLogger('ptxlogger')
 
 class Target():
-    def __init__(self,
-                 xml_element=None,
-                 name=None,
-                 format=None,
-                 source=None,
-                 publication=None,
-                 output_dir=None,
-                 stringparams=None,
-                 xsl_path=None,
-                 xmlid_root=None,
-                 project_path=None,
-                 pdf_method=None):
-        if project_path is None:
-            project_path = Path()
-        if xml_element is None:
-            template_xml = static.path("templates","project.ptx")
-            xml_element = ET.parse(template_xml).getroot().find("targets/target")
-            publication_path = static.path("templates","publication.ptx")
-            for pub_ele in xml_element.xpath("publication"):
-                pub_ele.text = publication_path
-        if xml_element.tag != "target":
-            raise ValueError("xml_element must have tag `target` as root")
-        # construct self.xml_element
-        # set name and pdf-method attributes
-        if name is not None:
-            xml_element.set("name",name)
-        if pdf_method is not None:
-            xml_element.set("pdf-method",pdf_method)
-        # set subelements with text nodes
-        tag_pairs = [
-            ("format",format),
-            ("source",source),
-            ("publication",publication),
-            ("output-dir",output_dir),
-            ("xsl",xsl_path),
-            ("xmlid-root",xmlid_root),
-        ]
-        for tag,ele_text in tag_pairs:
-            if ele_text is not None:
-                for tag_element in xml_element.xpath(tag):
-                    xml_element.remove(tag_element)
-                tag_element = ET.SubElement(xml_element,tag)
-                tag_element.text = ele_text.strip()
-        # set several stringparam subelements with key/value attributes
-        if stringparams is not None:
-            for sp_element in xml_element.xpath("stringparam"):
-                xml_element.remove(sp_element)
-            for key,val in stringparams.items():
-                sp_element = ET.SubElement(xml_element,"stringparam")
-                sp_element.set("key",key.strip())
-                sp_element.set("value", val.strip())
+    def __init__(self,xml_element,project_path):
         # construction is done!
         self.__xml_element = xml_element
-        self.__project_path = project_path
+        self.__project_path = Path(project_path).resolve()
+        # ensure assets directories exist as assumed by core PreTeXt
+        if self.external_dir() is not None:
+            os.makedirs(self.external_dir(), exist_ok=True)
+        if self.generated_dir() is not None:
+            os.makedirs(self.generated_dir(), exist_ok=True)
 
-    def xml_element(self):
+    def xml_element(self) -> Element:
         return self.__xml_element
 
+    def project_path(self):
+        return self.__project_path
+
     def name(self) -> str:
         return self.xml_element().get("name").strip()
 
@@ -82,7 +42,7 @@ class Target():
         return self.xml_element().find("format").text.strip()
 
     def source(self) -> Path:
-        return (Path(self.__project_path)/self.xml_element().find("source").text.strip()).resolve()
+        return (self.project_path()/self.xml_element().find("source").text.strip())
 
     def source_dir(self) -> Path:
         return Path(self.source()).parent
@@ -93,7 +53,7 @@ class Target():
         return ele_tree.getroot()
 
     def publication(self) -> Path:
-        return (Path(self.__project_path)/self.xml_element().find("publication").text.strip()).resolve()
+        return (self.project_path()/self.xml_element().find("publication").text.strip())
 
     def publication_dir(self) -> Path:
         return self.publication().parent
@@ -106,7 +66,7 @@ class Target():
         ele_tree.xinclude()
         return ele_tree.getroot()
 
-    def external_dir(self) -> Path:
+    def external_dir(self) -> Optional[Path]:
         dir_ele = self.publication_xml().find("source/directories")
         if dir_ele is None:
             log.error("Publication file does not specify asset directories.")
@@ -114,7 +74,7 @@ class Target():
         rel_dir = dir_ele.get("external")
         return self.source_dir()/rel_dir
 
-    def generated_dir(self) -> Path:
+    def generated_dir(self) -> Optional[Path]:
         dir_ele = self.publication_xml().find("source/directories")
         if dir_ele is None:
             log.error("Publication file does not specify asset directories.")
@@ -124,6 +84,13 @@ class Target():
 
     def output_dir(self) -> Path:
         return (Path(self.__project_path)/self.xml_element().find("output-dir").text.strip()).resolve()
+    
+    def port(self) -> int:
+        view_ele = self.xml_element().find("view")
+        if view_ele is not None and view_ele.get("port") is not None:
+            return int(view_ele.get("port"))
+        else:
+            return 8000
 
     def stringparams(self):
         return {
@@ -147,33 +114,15 @@ class Target():
 
 
 class Project():
-    def __init__(self,
-                 xml_element=None,
-                 targets=None,
-                 project_path=None):
-        if project_path is None:
-            project_path = Path()
-        if xml_element is None:
-            if utils.project_path() is not None:
-                xml_element = ET.parse(utils.project_path()/"project.ptx").getroot()
-            else:
-                template_xml = static.path("templates","project.ptx")
-                xml_element = ET.parse(template_xml).getroot()
-        else:
-            if xml_element.tag != "project":
-                raise ValueError("xml_element must have tag `project` as root")
-        if targets is not None:
-            for targets_element in xml_element.xpath("targets"):
-                xml_element.remove(targets_element)
-            targets_element = ET.SubElement(xml_element,"targets")
-            for target in targets:
-                targets_element.append(target.xml_element())
+    def __init__(self,project_path=None):
+        project_path = project_path or utils.project_path()
+        xml_element = ET.parse(project_path/"project.ptx").getroot()
         self.__xml_element = xml_element
         self.__project_path = project_path
-        # prepre core PreTeXt pythons scripts
+        # prepre core PreTeXt python scripts
         self.init_ptxcore()
 
-    def xml_element(self):
+    def xml_element(self) -> Element:
         return self.__xml_element
 
     def targets(self):
@@ -192,11 +141,11 @@ class Project():
         else:
             target_element=self.xml_element().find(f'targets/target[@name="{name}"]')
         if target_element is not None:
-            return Target(xml_element=target_element)
+            return Target(xml_element=target_element,project_path=self.__project_path)
         else:
             return None
 
-    def view(self,target_name,access,port,watch=False):
+    def view(self,target_name:str,access:str,port:int,watch:bool=False):
         target = self.target(target_name)
         directory = target.output_dir()
         if watch:
@@ -205,12 +154,16 @@ class Project():
             watch_directory = None
         if not target.output_dir().exists():
             log.error(f"The directory `{target.output_dir()}` does not exist.")
-            log.error(f"Run `pretext build {target}` to build your project before viewing.")
+            log.error(f"Run `pretext view {target.name()} -b` to build your project before viewing.")
             return
         watch_callback=lambda:self.build(target_name)
         utils.run_server(directory,access,port,watch_directory,watch_callback)
 
     def build(self,target_name,clean=False):
+        target = self.target(target_name)
+        if target is None:
+            log.error(f"Target {target_name} not found.")
+            return
         # Check for xml syntax errors and quit if xml invalid:
         if not self.xml_source_is_valid(target_name):
             return
@@ -218,10 +171,6 @@ class Project():
             return
         # Validate xml against schema; continue with warning if invalid:
         self.xml_schema_validate(target_name)
-        # Ensure directories for assets and generated assets to avoid errors when building:
-        target = self.target(target_name)
-        os.makedirs(target.external_dir(), exist_ok=True)
-        os.makedirs(target.generated_dir(), exist_ok=True)
         if clean:
             # refuse to clean if output is not a subdirectory of the working directory or contains source/publication
             if Path(self.__project_path) not in target.output_dir().parents:
@@ -241,7 +190,7 @@ class Project():
             temp_xsl_path = Path(temp_xsl_dir)
             if target.xsl_path() is not None:
                 log.info(f'Building with custom xsl {target.xsl_path()} specified in project.ptx')
-                utils.copy_expanded_xsl(target.xsl_path(), temp_xsl_dir)
+                utils.copy_expanded_xsl(target.xsl_path(), temp_xsl_path)
                 custom_xsl = temp_xsl_path/target.xsl_path().name
             log.info(f"Preparing to build into {target.output_dir()}.")
             try:
@@ -259,18 +208,21 @@ class Project():
                 log.critical(
                     f"A fatal error has occurred:\n {e} \nFor more info, run pretext with `-v debug`")
                 log.debug(f"Critical error info:\n****\n", exc_info=True)
-                return
+                sys.exit(f"Failed to build pretext target {target.format()}.  Exiting...")
             # build was successful
             log.info(f"\nSuccess! Run `pretext view {target.name()}` to see the results.\n")
     
-    def generate(self,target_name,asset_list=None,all_formats=False):
+    def generate(self,target_name,asset_list=None,all_formats=False,xmlid=None):
         if asset_list is None:
             asset_list = []
             gen_all = True
         else:
             gen_all = False
         target = self.target(target_name)
-        os.makedirs(target.generated_dir(), exist_ok=True)
+        xmlid = xmlid or target.xmlid_root()
+        if target is None:
+            log.error(f"Target `{target_name}` not found.")
+            return
         #build targets:
         if gen_all or "webwork" in asset_list:
             webwork_output = target.generated_dir()/'webwork'
@@ -280,32 +232,32 @@ class Project():
         if gen_all or "latex-image" in asset_list:
             generate.latex_image(
                 target.source(), target.publication(), target.generated_dir(), target.stringparams(), 
-                target.format(), target.xmlid_root(), target.pdf_method(), all_formats
+                target.format(), xmlid, target.pdf_method(), all_formats
             )
         if gen_all or "asymptote" in asset_list:
             generate.asymptote(
                 target.source(), target.publication(), target.generated_dir(), target.stringparams(), 
-                target.format(), target.xmlid_root(), all_formats
+                target.format(), xmlid, all_formats
             )
         if gen_all or "sageplot" in asset_list:
             generate.sageplot(
                 target.source(), target.publication(), target.generated_dir(), target.stringparams(), 
-                target.format(), target.xmlid_root(), all_formats
+                target.format(), xmlid, all_formats
             )
         if gen_all or "interactive" in asset_list:
             generate.interactive(
                 target.source(), target.publication(), target.generated_dir(), target.stringparams(), 
-                target.xmlid_root(),
+                xmlid,
             )
         if gen_all or "youtube" in asset_list:
             generate.youtube(
                 target.source(), target.publication(), target.generated_dir(), target.stringparams(), 
-                target.xmlid_root(),
+                xmlid,
             )
         if gen_all or "codelens" in asset_list:
             generate.codelens(
                 target.source(), target.publication(), target.generated_dir(), target.stringparams(), 
-                target.xmlid_root(),
+                xmlid,
             )
 
     def deploy(self,target_name,update_source):
